#pragma once

#include <stdexcept>
#include <string>
#include <vector>

// we will represent balances as longs, account numbers as unsigned longs,
// and times as ints.
using balance_t = long;
using acct_no_t = unsigned long;
using timestamp_t = int;

struct Bank_error : public std::runtime_error
{
    Bank_error(const std::string& msg) : runtime_error(msg) { }
};

// Represents a single bank account, with account number, balance, and owner.
struct Account
{
    acct_no_t id;
    balance_t balance;
    std::string owner;
};

// There are two types of transactions: debit and credit:
enum class Transaction_type { debit, credit };

// A transaction involves a single account, and debits or credits a given
// amount.
struct Transaction
{
    acct_no_t acct_id;
    balance_t amount;
    Transaction_type type;
    timestamp_t timestamp;
};

// A ledger contains a vector of accounts and a vector of uncleared
// transactions (that is, transactions that haven't been executed against the
// accounts yet). The accounts are stored in order of increasing account number.
struct Ledger {
    std::vector<Account> accounts;
    std::vector<Transaction> transactions;
};

// Finds the account with the given account number.
Account& find_account(Ledger&, acct_no_t);

// Finds the account with the given account number.
const Account& find_account(const Ledger&, acct_no_t);

// Executes the given transaction on the given account.
// PRECONDITION: the account numbers must match
void execute_transaction(Account&, const Transaction&);

// Executes all transactions in the ledger and clears them from the ledger.
void clear_transactions(Ledger&);
